#ifndef __REGIONSTATS_H__
#define __REGIONSTATS_H__

#include "GenomeSequence.h"
#include "SamFlag.h"
#include "SamFile.h"
#include  "StringArray.h"
#include "InputFile.h"
#include <map>
#include <cmath>

class Stats
{
 public:
  uint64_t nReads;
  int segCount;
  uint32_t totalLen;
  uint32_t atCnt;
  uint32_t gcCnt;
  double gcContent;
  
 public:
  Stats(){
    nReads = 0;
    segCount = 0;
    totalLen = 0;
    atCnt=gcCnt=0;
    gcContent = 0.0;
  }
};

class RegionStats
{
 public:
  GenomeSequence referencegenome;
  std::vector<String> contigs;
  std::map<String, vector< std::pair<int, int> > > genomeRegions;
  std::map<String, vector<String> > genomeRegions_lines;
  std::map<String, vector<String> > genomeRegionGroups;
  std::map<String, Stats> genomeRegionStats;
  std::map<int, uint64_t> regionReadCountInGCBins;
  std::map<int, uint64_t> geneReadCountInGCBins;
  std::map<String, Stats> geneStats;
  int minOverlapLen;
  int minMapQuality;
  uint64_t nReads;
  uint64_t nMapped2Targets;
  uint64_t nTotalMappedReads;
  bool norm_by_mapped2target;
  bool uniq;
  bool fully_contained;

public:
  RegionStats();
  RegionStats(String &);
  ~RegionStats();
  void Init();
  void SetUniqFlag(bool);
  void SetMinOverlapLen(int overlapLen) { minOverlapLen = overlapLen; } 
  void SetFullyContainedFlag(bool flag) { fully_contained = flag; } 
  void SetMinMapQuality(int mapq) { minMapQuality = mapq; }
  void SetNormalization(bool norm) { norm_by_mapped2target = norm; }
  void LoadRegionList(String &);
  void LoadReferenceGenome(String &);
  void CalcRegionGCContent();
  double CalcRegionGCContent(GenomeSequence &ref, uint32_t, uint32_t, int &, int &);
  void CalcRegionReadCountInGCBins();
  void Reset();
  void OutputRegionStats(String &outFile);
  void OutputGeneStats(String &outFile);
  int CalcRegionStats(String &bamFiles);
  void CalcRegionStats(StringArray &bamFiles);
  void CalcGeneGCContent();
  int toGCBin(double);
  int CountMappedReads(String&);
};

#endif

