#include "RegionStats.h"
#include "StringArray.h"
#include "Parameters.h"

int main(int argc, char *argv[])
{
   String regionsInputFile = "/home/bingshan/data/db/refFlat.tbl/refFlat.exons.v37.tbl";
   String regionsOutFile, genesOutFile;
   StringArray bamFiles;
   String reference = "/data/local/ref/karma.ref/human.g1k.v37";
   int overlapLen = 1;
   int minMapQual = 0;
   bool uniq_count = false;
   bool norm_by_mapped2target = false;
   //bool fully_contained = false;

  ParameterList pl;

  BEGIN_LONG_PARAMETERS(longParameters)
    LONG_PARAMETER_GROUP("Input Files")
      LONG_STRINGPARAMETER("reference",&reference)  
      LONG_STRINGPARAMETER("regions",&regionsInputFile)
    LONG_PARAMETER_GROUP("Mapping filters")
      LONG_INTPARAMETER("min_overlap",&overlapLen)
      //LONG_PARAMETER("fully_contained",&fully_contained)
      LONG_INTPARAMETER("min_map_quality",&minMapQual)
    LONG_PARAMETER_GROUP("Normalization")
      LONG_PARAMETER("norm_by_mapped2target",&norm_by_mapped2target)
    LONG_PARAMETER_GROUP("Output file")
      LONG_STRINGPARAMETER("exon_out",&regionsOutFile)
      LONG_STRINGPARAMETER("gene_out",&genesOutFile)
      LONG_PARAMETER("uniq",&uniq_count)
   END_LONG_PARAMETERS();

  pl.Add(new LongParameters("", longParameters));
  //pl.Read(argc, argv);
  int in = pl.ReadWithTrailer(argc, argv, 1);
  for (int i=in+1; i<argc; i++){
        bamFiles.Push(argv[i]);
  }
  pl.Status();

  printf("NOTE: start positions of regions are 1-based!\n\n");

  if(reference.Length()==0)
	error("Reference is not provided!\n");

  if(regionsInputFile.Length()==0)
  	error("Input file of list of regions is not provided!\n");

  if(bamFiles.Length()==0 && regionsOutFile.Length()>0)
  	error("Input BAM is not provided!\n");

  if(regionsOutFile.Length()==0 && genesOutFile.Length()==0)
   error("No output files provided!\n");


  RegionStats regions;

  regions.SetUniqFlag(uniq_count);
  regions.SetMinOverlapLen(overlapLen);
  //regions.SetFullyContainedFlag(fully_contained);
  regions.SetMinMapQuality(minMapQual);
  regions.SetNormalization(norm_by_mapped2target);
  regions.LoadRegionList(regionsInputFile);
  regions.LoadReferenceGenome(reference);
  regions.CalcRegionStats(bamFiles);
  regions.CalcGeneGCContent();
  if(genesOutFile.Length()>0) regions.OutputGeneStats(genesOutFile);
  if(regionsOutFile.Length()>0) regions.OutputRegionStats(regionsOutFile);

  return(0);
} //END of main
